<?php
/**
 * Plugin Name: AudioIgniter Pro
 * Plugin URI: http://www.cssigniter.com/ignite/plugins/audioigniter-pro/
 * Description: Pro add-ons for AudioIgniter.
 * Author: The CSSIgniter Team
 * Author URI: http://www.cssigniter.com
 * Version: 1.1.0
 * Text Domain: audioigniter-pro
 * Domain Path: languages
 *
 * AudioIgniter Pro is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * AudioIgniter Pro is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AudioIgniter Pro. If not, see <http://www.gnu.org/licenses/>.
 *
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class AudioIgniter_Pro {

	/**
	 * AudioIgniter Pro version.
	 *
	 * @var string
	 */
	public static $version = '1.1.0';

	/**
	 * Instance of this class.
	 *
	 * @var AudioIgniter_Pro
	 * @since 1.0.1
	 */
	protected static $instance = null;


	/**
	 * AudioIgniter Pro Instance.
	 *
	 * Instantiates or reuses an instance of AudioIgniter Pro.
	 *
	 * @since 1.0.0
	 * @static
	 * @see AudioIgniter_Pro()
	 * @return AudioIgniter_Pro - Single instance.
	 */
	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * The URL directory path (with trailing slash) of the main plugin file.
	 *
	 * @var string
	 * @since 1.0.0
	 */
	protected static $plugin_url = '';

	/**
	 * The filesystem directory path (with trailing slash) of the main plugin file.
	 *
	 * @var string
	 * @since 1.0.0
	 */
	protected static $plugin_path = '';


	/**
	 * AudioIgniter constructor. Intentionally left empty so that instances can be created without
	 * re-loading of resources (e.g. scripts/styles), or re-registering hooks.
	 * http://wordpress.stackexchange.com/questions/70055/best-way-to-initiate-a-class-in-a-wp-plugin
	 * https://gist.github.com/toscho/3804204
	 *
	 * @since 1.0.0
	 */
	public function __construct() {}


	/**
	 * Kickstarts plugin loading.
	 *
	 * @since 1.0.0
	 */
	public function plugin_setup() {
		self::$plugin_url  = plugin_dir_url( __FILE__ );
		self::$plugin_path = plugin_dir_path( __FILE__ );

		load_plugin_textdomain( 'audioigniter-pro', false, dirname( self::plugin_basename() ) . '/languages' );

		add_action( 'audioigniter_init', array( $this, 'init' ) );
		add_action( 'audioigniter_admin_init', array( $this, 'admin_init' ) );

		if ( ! class_exists( 'AudioIgniter' ) ) {
			add_action( 'admin_notices', 'AudioIgniter_Pro::notice_plugin_required' );
		}
	}

	/**
	 * Registers actions that need to be run on both admin and frontend
	 *
	 * @since 1.0.0
	 */
	public function init() {
		add_action( 'init', array( $this, 'register_taxonomies' ) );
		add_action( 'init', array( $this, 'register_scripts' ) );
		add_action( 'vc_before_init', array( $this, 'register_vc_shortcodes' ) );
		add_filter( 'audioigniter_register_widgets', array( $this, 'register_widgets' ) );

		include_once( 'class-audioigniter-customizer.php' );
		new AudioIgniter_Customizer();

		include_once( 'class-audioigniter-updater.php' );
		new AudioIgniter_Updater();

		add_filter( 'audioigniter_shortcode_data_attributes_array', array( $this, 'filter_shortcode_data_attributes_array' ), 10, 3 );
		add_filter( 'audioigniter_shortcode_shortcircuit', array( $this, 'filter_shortcode_shortcircuit' ), 10, 4 );
		add_filter( 'audioigniter_player_types', array( $this, 'filter_player_types' ) );
		add_filter( 'audioigniter_metabox_tracks_show_upgrade_button', '__return_false' );

		add_action( 'wp_footer', array( $this, 'global_footer_player' ) );
	}

	/**
	 * Registers actions that need to be run on admin only.
	 *
	 * @since 1.0.0
	 */
	public function admin_init() {
		if ( ! is_admin() ) {
			return;
		}

		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_scripts' ) );

		add_filter( 'audioigniter_metabox_tracks_container_classes', array( $this, 'filter_metabox_tracks_container_classes' ) );
		add_filter( 'audioigniter_metabox_track_classes', array( $this, 'filter_metabox_track_classes' ), 10, 2 );
		add_filter( 'audioigniter_metabox_tracks_footer_links', array( $this, 'filter_metabox_tracks_footer_links' ), 10 );
		add_action( 'audioigniter_metabox_tracks_field_controls', array( $this, 'action_metabox_tracks_field_controls' ) );
		add_action( 'audioigniter_metabox_tracks_repeatable_track_field_before_title', array( $this, 'action_metabox_tracks_move_handle' ) );
		add_action( 'audioigniter_metabox_tracks_repeatable_track_field_after_track_upload_button', array( $this, 'action_metabox_tracks_soundcloud_sync_button' ) );

		add_action( 'wp_ajax_ai_upload_image', array( $this, 'ajax_upload_image' ) );

		add_action( 'save_post', array( $this, 'invalidate_transient_global_footer_player_id' ) );
		add_action( 'delete_post', array( $this, 'invalidate_transient_global_footer_player_id' ) );
	}

	public function register_taxonomies() {
		$labels = array(
			'name'              => esc_html_x( 'Playlist Categories', 'taxonomy general name', 'audioigniter-pro' ),
			'singular_name'     => esc_html_x( 'Playlist Category', 'taxonomy singular name', 'audioigniter-pro' ),
			'search_items'      => esc_html__( 'Search Playlist Categories', 'audioigniter-pro' ),
			'all_items'         => esc_html__( 'All Playlist Categories', 'audioigniter-pro' ),
			'parent_item'       => esc_html__( 'Parent Playlist Categories', 'audioigniter-pro' ),
			'parent_item_colon' => esc_html__( 'Parent Playlist Categories:', 'audioigniter-pro' ),
			'edit_item'         => esc_html__( 'Edit Playlist Category', 'audioigniter-pro' ),
			'update_item'       => esc_html__( 'Update Playlist Category', 'audioigniter-pro' ),
			'add_new_item'      => esc_html__( 'Add New Playlist Category', 'audioigniter-pro' ),
			'new_item_name'     => esc_html__( 'New Playlist Category Name', 'audioigniter-pro' ),
			'menu_name'         => esc_html__( 'Categories', 'audioigniter-pro' ),
			'view_item'         => esc_html__( 'View Playlist Category', 'audioigniter-pro' ),
			'popular_items'     => esc_html__( 'Popular Playlist Categories', 'audioigniter-pro' ),
		);

		register_taxonomy( 'ai_playlist_category', array( AudioIgniter()->post_type ), array(
			'labels'            => $labels,
			'hierarchical'      => true,
			'public'            => false,
			'show_ui'           => true,
			'show_admin_column' => true,
			'show_in_menu'      => true,
			'show_in_nav_menus' => false,
			'rewrite'           => array( 'slug' => _x( 'playlist-category', 'taxonomy slug', 'audioigniter-pro' ) ),
			'description'       => esc_html__( 'Used solely for better playlist organization.', 'audioigniter-pro' ),
		) );
	}

	public function register_vc_shortcodes() {
		require_once( 'class-audioigniter-visual-composer-playlist.php' );
		$vc_shortcode = new AudioIgniter_Visual_Composer_Playlist();
		$vc_shortcode->load();
	}

	public function register_widgets( $widgets ) {
		return array_merge( $widgets, array(
			'AudioIgniter_Playlist_Widget' => $this->plugin_path() . 'class-audioigniter-playlist-widget.php',
		) );
	}


	/**
	 * Register (but not enqueue) all scripts and styles to be used throughout the plugin.
	 *
	 * @since 1.0.0
	 */
	public function register_scripts() {
		wp_register_script( 'audioigniter-pro-admin', $this->plugin_url() . 'assets/js/audioigniter-pro.js',	array(
			'jquery-ui-core',
			'jquery-ui-sortable',
		), self::$version, true );

		wp_localize_script( 'audioigniter-pro-admin', 'ai_pro_scripts', array(
			'ajax_url' => admin_url( 'admin-ajax.php' ),
			'_ai_upload_nonce' => wp_create_nonce( 'audioigniter-image-upload' ),
			'sc_client_id' => get_option( 'audioigniter_sc_client_id' ),
			'messages' => array(
				'media_title_batch' => esc_html__( 'Select or upload audio media', 'audioigniter-pro' ),
				'error_intro' => esc_html__( 'There was an error while fetching data from SoundCloud.', 'audioigniter-pro' ),
				'error_missing_client_id' => esc_html__( 'Make sure your SoundCloud client ID is set in the customizer.', 'audioigniter-pro' ),
				'error_disabled_embed' => esc_html__( 'Most probably the track owner has disabled external embedding.', 'audioigniter-pro' ),
				'error_track_support' => esc_html__( 'Currently only single tracks are supported by the AudioIgniter player.', 'audioigniter-pro' ),
			),
		) );
	}

	/**
	 * Enqueues admin scripts and styles.
	 *
	 * @since 1.0.0
	 */
	public function enqueue_admin_scripts( $hook ) {
		$screen = get_current_screen();

		if ( 'post' === $screen->base && AudioIgniter()->post_type === $screen->post_type ) {
			wp_enqueue_script( 'audioigniter-pro-admin' );
		}
	}

	/**
	 * Downloads an image from the specified URL and attaches it to a post.
	 *
	 * This is copied and edited from wp-admin/includes/media.php with original function name media_sideload_image() as of WP v4.5.3
	 * A $return option for 'id' has been added.
	 *
	 * @since 2.6.0
	 * @since 4.2.0 Introduced the `$return` parameter.
	 *
	 * @param string $file    The URL of the image to download.
	 * @param int    $post_id The post ID the media is to be associated with.
	 * @param string $desc    Optional. Description of the image.
	 * @param string $return  Optional. Accepts 'html' (image tag html) or 'src' (URL). Default 'html'.
	 * @return string|WP_Error Populated HTML img tag on success, WP_Error object otherwise.
	 */
	protected function media_sideload_image( $file, $post_id, $desc = null, $return = 'html' ) {
		if ( ! empty( $file ) ) {

			// Set variables for storage, fix file filename for query strings.
			preg_match( '/[^\?]+\.(jpe?g|jpe|gif|png)\b/i', $file, $matches );
			if ( ! $matches ) {
				return new WP_Error( 'image_sideload_failed', __( 'Invalid image URL', 'audioigniter-pro' ) );
			}

			$file_array = array();
			$file_array['name'] = basename( $matches[0] );

			// Download file to temp location.
			$file_array['tmp_name'] = download_url( $file );

			// If error storing temporarily, return the error.
			if ( is_wp_error( $file_array['tmp_name'] ) ) {
				return $file_array['tmp_name'];
			}

			// Do the validation and storage stuff.
			$id = media_handle_sideload( $file_array, $post_id, $desc );

			// If error storing permanently, unlink.
			if ( is_wp_error( $id ) ) {
				@unlink( $file_array['tmp_name'] );
				return $id;
			}

			$src = wp_get_attachment_url( $id );
		}

		// Finally, check to make sure the file has been saved, then return the HTML.
		if ( ! empty( $src ) ) {
			if ( $return === 'src' ) {
				return $src;
			}

			if ( $return === 'id' ) {
				return $id;
			}

			$alt = isset( $desc ) ? esc_attr( $desc ) : '';
			$html = "<img src='$src' alt='$alt' />";
			return $html;
		} else {
			return new WP_Error( 'image_sideload_failed' );
		}
	}

	public function ajax_upload_image() {
		if ( empty( $_POST ) || ! isset( $_POST['url'] ) ) { // Input var okay.
			wp_send_json_error( __( 'Image URL is required.', 'audioigniter-pro' ) );
		}

		if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'audioigniter-image-upload' ) ) { // Input var okay.
			wp_send_json_error( __( 'Not authorized.', 'audioigniter-pro' ) );
		}

		global $post;
		$image_url = esc_url_raw( $_POST['url'] ); // Input var okay.
		$image_id  = $this->media_sideload_image( $image_url, $post->ID, null, 'id' );

		if ( is_wp_error( $image_id ) ) {
			wp_send_json_error( __( 'Image could not be uploaded.', 'audioigniter-pro' ) );
		}

		$src = wp_get_attachment_thumb_url( $image_id );

		$response = array(
			'id'  => $image_id,
			'url' => $src,
		);

		wp_send_json_success( $response );
	}

	public function filter_metabox_tracks_container_classes( $classes ) {
		// Enable sortable for the repeatable tracks.
		$classes[] = 'ai-fields-sortable';

		return $classes;
	}

	public function filter_metabox_track_classes( $classes, $url ) {
		if ( strpos( $url, 'soundcloud.com' ) !== false ) {
			$classes[] = 'ai-soundcloud-track';
		}

		return $classes;
	}

	public function filter_metabox_tracks_footer_links( $links ) {
		unset( $links['rate_plugin'] );

		$new_links = array(
			'support'       => array(
				'title' => __( 'Pro Support', 'audioigniter-pro' ),
				'url'   => 'http://www.cssigniter.com/support/viewforum.php?f=105',
			),
			'documentation' => array(
				'title' => __( 'Documentation', 'audioigniter-pro' ),
				'url'   => 'http://www.cssigniter.com/docs/audioigniter/',
			),
		);

		$links = array_merge( $links, $new_links );

		return $links;
	}

	public function action_metabox_tracks_field_controls() {
		?>
		<button type="button" class="button ai-add-field-batch">
			<span class="dashicons dashicons-upload"></span>
			<?php esc_html_e( 'Batch Upload', 'audioigniter-pro' ); ?>
		</button>
		<?php
	}

	public function action_metabox_tracks_move_handle() {
		?>
		<span class="ai-field-sort-handle">
			<span class="screen-reader-text"><?php esc_html_e( 'Move track', 'audioigniter-pro' ); ?></span>
			<span class="dashicons dashicons-move"></span>
		</span>
		<?php
	}

	public function action_metabox_tracks_soundcloud_sync_button() {
		?>
		<button type="button" class="button ai-sync-soundcloud">
			<?php esc_html_e( 'Sync SoundCloud', 'audioigniter-pro' ); ?>
		</button>
		<?php
	}

	public function filter_shortcode_data_attributes_array( $data, $id, $post ) {
		$client_id = get_option( 'audioigniter_sc_client_id' );
		if ( ! empty( $client_id ) ) {
			$data['data-soundcloud-client-id'] = $client_id;
		}

		return $data;
	}

	/**
	 * Adds additional player types to AudioIgniter.
	 *
	 * @version 1.1.0
	 * @since   1.1.0
	 *
	 * @param array $player_types Player types array.
	 *
	 * @return array
	 */
	public function filter_player_types( $player_types ) {
		$player_types = array_merge( $player_types, array(
			'global-footer' => array(
				'label'      => __( 'Global Footer Player', 'audioigniter-pro' ),
				'no-support' => array(
					'show_credit',
					'max_width',
				),
				'info'       => __( "You've set this player as Global and it will appear at the bottom of your website, on every page. Please note that in the case that you've set multiple Global Players only the most recent one will be displayed.", 'audioigniter-pro' ),
			),
		) );

		return $player_types;
	}

	/**
	 * Determines if the [ai_playlist] shortcode should be short-circuited.
	 *
	 * @version 1.1.0
	 * @since   1.1.0
	 *
	 * @param bool    $shortcircuit Whether to short-circuit the shortcode.
	 * @param int     $id           The playlist ID.
	 * @param WP_Post $post         The playlist WP_Post object.
	 * @param array   $params       The playlist data attributes array.
	 *
	 * @return bool
	 */
	public function filter_shortcode_shortcircuit( $shortcircuit, $id, $post, $params ) {
		if ( ! empty( $params['data-player-type'] ) && 'global-footer' === $params['data-player-type'] ) {
			return true;
		}

		return $shortcircuit;
	}

	/**
	 * Return the post ID of the playlist that should be displayed in the global footer player.
	 *
	 * @version 1.1.0
	 * @since   1.1.0
	 *
	 * @return int|false
	 */
	public function get_global_footer_player_id() {

		$transient = get_transient( 'audioigniter_global_footer_player_id' );

		if ( false === $transient ) {
			$q = new WP_Query( array(
				'post_type'      => AudioIgniter()->post_type,
				'posts_per_page' => 1,
				'meta_query'     => array(
					array(
						'key'   => '_audioigniter_player_type',
						'value' => 'global-footer',
					),
				),
			) );

			$transient = array(
				'id' => false,
			);

			if ( $q->have_posts() ) {
				while ( $q->have_posts() ) {
					$q->the_post();
					$transient['id'] = get_the_ID();
					break;
				}
				wp_reset_postdata();
			}

			// Store array instead of id, as id may be false and we won't be able to distinguish from when the transient doesn't exist.
			// Non-expiring transient. It gets invalidated on post save.
			set_transient( 'audioigniter_global_footer_player_id', $transient );
		}

		return $transient['id'];
	}

	/**
	 * Echoes the output for the global footer player (most recent if multiples exist).
	 *
	 * @version 1.1.0
	 * @since   1.1.0
	 */
	public function global_footer_player() {
		$id = $this->get_global_footer_player_id();

		if ( empty( $id ) || ! AudioIgniter()->is_playlist( $id ) ) {
			return;
		}

		$post = get_post( $id );

		$params = apply_filters( 'audioigniter_global_footer_player_data_attributes_array', AudioIgniter()->get_playlist_data_attributes_array( $id ), $id, $post );
		$params = array_filter( $params, array( AudioIgniter()->sanitizer, 'array_filter_empty_null' ) );
		$params = AudioIgniter()->sanitizer->html_data_attributes_array( $params );

		// Returning a truthy value from the filter, will short-circuit execution of the global footer player.
		if ( false !== apply_filters( 'audioigniter_global_footer_player_shortcircuit', false, $id, $post, $params ) ) {
			return;
		}

		$data = '';
		foreach ( $params as $attribute => $value ) {
			$data .= sprintf( '%s="%s" ', sanitize_key( $attribute ), esc_attr( $value ) );
		}

		$output = sprintf( '<div id="audioigniter-%s" class="audioigniter-root" %s></div>',
			esc_attr( $id ),
			$data
		);

		echo $output; // WPCS: XSS ok.
	}

	public function invalidate_transient_global_footer_player_id( $post_id ) {
		delete_transient( 'audioigniter_global_footer_player_id' );
	}


	public static function notice_plugin_required() {
	    ?>
	    <div class="error notice">
	        <p>
		        <?php echo wp_kses(
			        sprintf(
		                /* translators: %s is a URL. */
				        __( 'AudioIgniter Pro requires AudioIgniter. Please install and activate the free <a href="%s" target="_blank">AudioIgniter</a> plugin.', 'audioigniter-pro' ),
				        'https://wordpress.org/plugins/audioigniter/'
			        ), array( 'a' => array( 'href' => true, 'target' => true ) )
		        ); ?>
	        </p>
	    </div>
	    <?php
	}

	public function plugin_activated() {
		if ( ! current_user_can( 'activate_plugins' ) ) {
			return;
		}

		if ( class_exists( 'AudioIgniter' ) ) {
			$this->register_taxonomies();
		}

		flush_rewrite_rules();
	}

	public function plugin_deactivated() {
		if ( ! current_user_can( 'activate_plugins' ) ) {
			return;
		}

		unregister_taxonomy( 'ai_playlist_category' );

		flush_rewrite_rules();
	}

	public static function plugin_basename() {
		return plugin_basename( __FILE__ );
	}

	public function plugin_url() {
		return self::$plugin_url;
	}

	public function plugin_path() {
		return self::$plugin_path;
	}
}

/**
 * Main instance of AudioIgniter_Pro.
 *
 * Returns the working instance of AudioIgniter_Pro. No need for globals.
 *
 * @since  1.0.0
 * @return AudioIgniter_Pro
 */
function AudioIgniter_Pro() {
	return AudioIgniter_Pro::instance();
}

add_action( 'plugins_loaded', array( AudioIgniter_Pro(), 'plugin_setup' ) );
register_activation_hook( __FILE__, array( AudioIgniter_Pro(), 'plugin_activated' ) );
register_deactivation_hook( __FILE__, array( AudioIgniter_Pro(), 'plugin_deactivated' ) );
