<?php
class AudioIgniter_Updater {

	public function __construct() {
		add_filter( 'pre_set_site_transient_update_plugins', array( $this, 'update_check_admin_handler' ), 10, 2 );
	}

	protected function get_plugin_object( $version, $package_url ) {
		$plugin              = new stdClass();
		$plugin->id          = dirname( AudioIgniter_Pro::plugin_basename() ); // This should be numeric. But what?
		$plugin->slug        = dirname( AudioIgniter_Pro::plugin_basename() );
		$plugin->url         = 'http://www.cssigniter.com/ignite/plugins/audioigniter-pro/';
		$plugin->new_version = $version;
		$plugin->package     = $package_url;

//		$plugin->tested                        = '4.6';
//		$plugin->compatibility                 = new stdClass();
//		$plugin->compatibility->scalar         = new stdClass();
//		$plugin->compatibility->scalar->scalar = false;

		return $plugin;
	}

	/**
	 * Action hook handler for theme updates checks. Intercepts theme update data before they are written into a transient.
	 * Don't use directly.
	 *
	 * Hooked on 'pre_set_site_transient_update_plugins' action hook.
	 *
	 * @param object $value An object containing the theme-update information returned by WordPress.org API.
	 * @param string $transient The transient name. 'update_plugins' in this case.
	 *
	 * @return object
	 */
	public function update_check_admin_handler( $value, $transient ) {
		$latest_version = $this->get_latest_version();
		$basename       = AudioIgniter_Pro::plugin_basename();
		$plugin         = $this->get_plugin_object( $latest_version, false );

		if ( $latest_version !== false ) {
			if ( version_compare( $latest_version, AudioIgniter_Pro::$version, '>' ) ) {
				$value->response[ $basename ] = $plugin;
			} else {
				unset( $plugin->tested );
				unset( $plugin->compatibility );
				$value->no_update[ $basename ] = $plugin;
			}
		}

		return $value;
	}

	public function get_latest_version() {
		$versions_url        = apply_filters( 'audioigniter_pro_latest_version_url', 'http://www.cssigniter.com/plugin_versions.json' );
		$update_period       = apply_filters( 'audioigniter_pro_update_period', 24 * 60 * 60 );
		$error_update_period = apply_filters( 'audioigniter_pro_update_period_after_error', 8 * 60 * 60 );
		$transient_name      = 'audioigniter_pro_latest_version';
		$plugin_versions     = '';
		$slug                = AudioIgniter_Pro::plugin_basename();

		if ( false === ( $latest_version = get_transient( $transient_name ) ) ) {
			$response = wp_remote_get( $versions_url );
			if ( is_wp_error( $response ) ) {
				set_transient( $transient_name, - 1, $error_update_period );

				return false;
			} else {
				if ( $response['response']['code'] == 200 ) {
					$plugin_versions = $response['body'];
				} else {
					set_transient( $transient_name, - 1, $error_update_period );

					return false;
				}
			}

			if ( empty( $plugin_versions ) ) {
				set_transient( $transient_name, - 1, $error_update_period );

				return false;
			}

			$plugin_versions = json_decode( $plugin_versions, true );

			if ( $plugin_versions === null || $plugin_versions === false ) {
				set_transient( $transient_name, - 1, $error_update_period );

				return false;
			}

			if ( ! isset( $plugin_versions[ $slug ] ) ) {
				set_transient( $transient_name, - 1, $error_update_period );

				return false;
			}

			$latest_version = $plugin_versions[ $slug ];

			set_transient( $transient_name, $latest_version, $update_period );
		}

		return $latest_version;
	}
}
